#ifndef DOACONTEXT_H_
#define DOACONTEXT_H_

/*
 * DOAContext.h 
 * Defines DOAContext class which has necessery global state for DOA to operate, 
 * as well as DOA-specific packet processing functions
 * 
 * @author levpopov, djwendel, evhan55
 * Code adapted from SampleOCDContext.h
 */

#include <map>
#include <string.h>
#include "DOAGlobals.h"

using namespace std;

class DOAOCD;
class DOAStateInfo;

class DOAContext
{
	/* map from incoming EID's to StateInfo instances */
    map<eid*, DOAStateInfo*, cmpEID> addrToState;

    /* Pointer to the OCD associated with this context. */
    DOAOCD* doaOCD;
    
    /* Previously Globals, now local variables so they can be loaded from config file. */

    uint16_t numEIDDelegates;
    char* EIDDelegates;
    uint16_t numIPDelegates;
    char* IPDelegates;
    char* myAddress;
    char* myIP;
    eid mEID;
    uint16_t myPortNum;
    	
public:
	/* DOA OCD listens for data on this connection. */
    int mySockFD;

	DOAContext(){};
	DOAContext ( DOAOCD* doaOCD, string fileName );

	/* initializes DOA Context - opens socket and sets up initial state */
	void init();
	
	/*returns the EID of the host on which OCD is running */ 
	eid* myEID();
	
	/* Sends a packet over DOA, over connection described by info */
	int sendDOA(DOAStateInfo* info, char* data, int len);
	
	/* The callback registered with the socket - will dispatch to appropriate
	 * receive function based on packet source and type */
	void receiveDOA();
				
	int setupTunnel(DOAStateInfo* info);

	void addRemoteHost (DOAStateInfo* currOCDState);
	DOAStateInfo* lookupRemoteHost (eid* addr);
	
	friend void registerWithDNS(DOAContext* context);
    friend void resolveEID(eid* inputEID, DOAStateInfo* info);
	friend void registerWithDHT(DOAContext* context);
};

static DOAContext text;

#endif /*DOACONTEXT_H_*/
