#ifndef DOASTATEINFO_H_
#define DOASTATEINFO_H_

#include "DOAOCD.h"
#include <vector>
#include <sys/types.h>
#include <sys/socket.h>

#include "OCD_include.h"
#include "DOAGlobals.h"

using namespace std;
/*
 * DOAStateInfo.h
 * Defines DOAStateInfo class which describes per-connection state required by
 * DOA.
 * 
 * @author levpopov, djwendel, evhan55
 * Adapted from SampleOCDStateInfo.h
 */

// To store packets, they are queued while this state is initializing.
struct QueuedPacket {
	int len;
	char* data;
};

class DOAOCD;

class DOAStateInfo: public OCDStateInfo 
{
	/* We need to keep a stack of EIDs somewhere here,
	 * periodically refreshed from DHT. Its a vector for now */
	vector<eid*> route;
	
	/* This vector holds the IP addresses of delegates on the route.
	 * It is sent in each header to eliminate DHT lookups by delegates,
	 * even for NEBs that do not store state.*/
	vector<uint32_t> ip_hints;
	
	// The final destination eid for this connection
	eid dest_eid;
	
	// The destination IP address for packets sent (= IP of first delegate in route)
	struct in_addr dest_ip;
	
	// This flag is false until the EIDs have been fully resolved and a next-hop IP has been found.
	bool readyFlag;
	bool resolvingFlag; //set to TRUE while DHT resolution is in process
	
	// This array of pointers is used as a buffer to store queued packets.
	QueuedPacket** packets;
	int numQPackets;
	
public:
	DOAStateInfo(LayerHandle myHandle, OCI* oci, DOAOCD *doaOCD);
	void receivedDNSReply (eid* address);
	void checkAndResolve ();
	void receivedDHTReply (char* erec_string);
	bool isReady();
	bool isResolving();
	void enqueuePacket(char* data, int len);
	~DOAStateInfo();
	
    friend class DOAOCD;
    friend class DOAContext;
    friend class DOAPacket;
    friend bool doaDNSRequest(char* url, DOAStateInfo* info);
    friend void resolveEID(eid* inputEID, DOAStateInfo* info);

private:
	void sendQueued();
	
};

#endif /*DOASTATEINFO_H_*/
