#ifndef DOADHT_H_
#define DOADHT_H_

/* DOADHT is the interface that takes care of resolving EIDs to ERecords. */

#include <netinet/udp.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "../DOAStateInfo.h"
#include "../DOAContext.h"
#include "../ERecord.h"

/* This just creates a registration packet for our DHT stub.  
 * We won't go into the details of the packet format because a real 
 * implementation would use ERecord.h and talk to a real DHT.
 * For a secure implementation, endhosts should use their public key 
 * to sign their ERecords before putting them into the DHT.  The DHT
 * should check that the ERecord is signed by the public key that hashes
 * to the EID.*/
void registerWithDHT(DOAContext* context) {
	// create erecord (actually, this code creates the "put" packet, which contains the erecord info)

	struct sockaddr_in tmpDestAddr;
	in_addr addr;
	addr.s_addr = inet_addr(DOA_DNS_DHT_IP);
	tmpDestAddr.sin_addr = addr;
	tmpDestAddr.sin_port = htons(DOA_DHT_PORT);
   
	tmpDestAddr.sin_family = AF_INET;
	memset(&(tmpDestAddr.sin_zero), '\0', 8);  // zero the rest of the struct
    	
	int dht_packet_len = 30;
	dht_packet_len += context->numEIDDelegates*20;
	dht_packet_len += context->numIPDelegates*4;

	char packet2[dht_packet_len];
	int packet_offset = 0;
	memcpy(packet2,&(context->mEID),EID_SIZE);
	packet_offset += EID_SIZE;
	memcpy(packet2+packet_offset,(char*)(&(context->numEIDDelegates)),2);
	packet_offset += 2;
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "now copying over EID Delegates.\n");
	memcpy(packet2+packet_offset,context->EIDDelegates,context->numEIDDelegates*20);
	packet_offset += context->numEIDDelegates*20;
	uint32_t IPDelegatesAddr;
	if (context->numIPDelegates > 0) {
		IPDelegatesAddr = inet_addr(context->IPDelegates);
	}
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "now copying over IP Delegates.\n");
	memcpy(packet2+packet_offset,(char*)(&IPDelegatesAddr),context->numIPDelegates*4);
	packet_offset += context->numIPDelegates*4;
	uint32_t myIPAddr = inet_addr(context->myIP);
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "now copying my IP address.\n");
	memcpy(packet2+packet_offset,(char*)(&myIPAddr),4);
	packet_offset += 4;
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "now copying TTL.\n");
	memset(packet2+packet_offset,'\0',4);

	// (no signing without security)
	// register erecord with DHT (via "put" packet)
	
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Sending DHT registration packet for: %s\n", context->myAddress);
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Packet length is %d\n",dht_packet_len);
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "DHT packet contents: \n%s\n\n", packet2);

	int numSent = sendto (context->mySockFD,
			      packet2, dht_packet_len, 
			      0, 
			      (struct sockaddr *)& (tmpDestAddr),
    			      sizeof (struct sockaddr_in));

}

/* This sends a get request to the DHT */
void resolveEID(eid* inputEID, DOAStateInfo* info){

  // create a packet with EID
  // send it out to DHT

  // SEND REQUEST PACKET TO DHT
  // format of packet: (EIDEIDEIDEIDEIDEIDEI)
  struct sockaddr_in tmpDestAddr;
  in_addr addr;
  addr.s_addr = inet_addr(DOA_DNS_DHT_IP);
  tmpDestAddr.sin_addr = addr;
  tmpDestAddr.sin_port = htons(DOA_DHT_PORT);
  
  tmpDestAddr.sin_family = AF_INET;
  memset(&(tmpDestAddr.sin_zero), '\0', 8);  // zero the rest of the struct
  
  char packet[EID_SIZE+4];
  memcpy(packet,inputEID,EID_SIZE);
  memcpy(packet+EID_SIZE,(char*)(&info),4);
  
  PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Sending DHT lookup request packet for: %s\n", inputEID);
  
  int numSent = sendto (((DOAOCD*)info->ocd)->context->mySockFD,
			packet,  EID_SIZE+4, 
			0, 
			(struct sockaddr *)& (tmpDestAddr),
			sizeof (struct sockaddr_in));
  
  // send again just in case
  numSent = sendto (((DOAOCD*)info->ocd)->context->mySockFD,
		    packet,  EID_SIZE+4, 
		    0, 
		    (struct sockaddr *)& (tmpDestAddr),
		    sizeof (struct sockaddr_in));
  
}

class DOADHT
{
 public:
  DOADHT();
  virtual ~DOADHT();
};

#endif /*DOADHT_H_*/
