#ifndef DOADNS_H_
#define DOADNS_H_

/* DOADNS is the interface that takes care of registration with DNS and resolving DOA urls to EIDs. */

#include <string.h>
#include <stdio.h>
#include <netinet/udp.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <openssl/dsa.h>

#include "../DOAStateInfo.h"
#include "../DOAContext.h"

/* Create the registration packet and send it to a fake DNS server
   Expected format of registration packet: (0someaddressEIDEIDEIDEIDEIDEIDEI) */
void registerWithDNS(DOAContext* context) {

	struct sockaddr_in tmpDestAddr;
	in_addr addr;
	addr.s_addr = inet_addr(DOA_DNS_DHT_IP);
	tmpDestAddr.sin_addr = addr;
	tmpDestAddr.sin_port = htons(DOA_DNS_PORT);
   
	tmpDestAddr.sin_family = AF_INET;
	memset(&(tmpDestAddr.sin_zero), '\0', 8);  // zero the rest of the struct
   	
	char packet[1+strlen(context->myAddress)+EID_SIZE];
	// first digit is a zero to indicate to the DNS server that this is a registration packet
	memset(packet,'0',1);
	// copy my address into the packet
	memcpy(packet+1,context->myAddress,strlen(context->myAddress));
	// copy my EID into the packet
	memcpy(packet+1+strlen(context->myAddress),&(context->mEID),EID_SIZE);
    
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Sending DNS registration packet for: %s\n", context->myAddress);
	PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Packet length is %d\n", 1+strlen(context->myAddress)+EID_SIZE);

	int numSent = sendto (context->mySockFD,
			      packet,  1+strlen(context->myAddress)+EID_SIZE, 
			      0, 
			      (struct sockaddr *)& (tmpDestAddr),
    			      sizeof (struct sockaddr_in));

	// send again just in case
	numSent = sendto (context->mySockFD,
			      packet,  1+strlen(context->myAddress)+EID_SIZE, 
			      0, 
			      (struct sockaddr *)& (tmpDestAddr),
    			      sizeof (struct sockaddr_in));

}

/* Performs DOA DNS lookup (translate url into eid) 
 * returns true iff request was made */ 
bool doaDNSRequest(char* url, DOAStateInfo* info){
	PRINT_DEBUG (DEBUG_LEVEL_SUPER,"doaDNSrequest...\n");
	
	eid reply;
	
	struct sockaddr_in tmpDestAddr;
	in_addr addr;
	addr.s_addr = inet_addr(DOA_DNS_DHT_IP);
    tmpDestAddr.sin_addr = addr;
    tmpDestAddr.sin_port = htons(21567);
   
    tmpDestAddr.sin_family = AF_INET;
    memset(&(tmpDestAddr.sin_zero), '\0', 8);  // zero the rest of the struct
   	
   	//Expected format of resolve packet: (dnsINFOsomeaddress)
    char packet[strlen(url)+7];
    memcpy(packet,"dns",3);
    memcpy(packet+3,(char*)(&info),4);
    memcpy(packet+7,url,strlen(url));
    
    PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Sending DNS lookup packet for: %s\n", url);
    PRINT_DEBUG(DEBUG_LEVEL_SUPER, "Packet length is %d\n", strlen(url)+7);
    PRINT_DEBUG(DEBUG_LEVEL_SUPER, "State Info = %d\n", &info);

    int numSent = sendto (((DOAOCD*)info->ocd)->context->mySockFD,
			  packet, strlen(url)+7, 0, (struct sockaddr *)& (tmpDestAddr),
    				    sizeof (struct sockaddr_in));

	return true;
}


class DOADNS
{
public:
	DOADNS();
};

#endif /*DOADNS_H_*/
